SystemOrganization addCategory: #'Comet-Core'!

!WAKom class methodsFor: '*comet' stamp: 'lr 4/9/2006 20:52'!
service
	^ service! !

!Socket methodsFor: '*comet-sending' stamp: 'lr 4/10/2006 21:56'!
pushData: aStringOrByteArray
	"Send aStringOrByteArray to the remote host trough the given socket. Push the data immediately and ignore all kinds of errors. Return true if all the data could be sent successfully, in case of problems destroy the socket and return false."

	| bytesSent bytesToSend count |
	bytesSent := 0.
	bytesToSend := aStringOrByteArray size.
	[ bytesSent < bytesToSend ] whileTrue: [
		count := self 
			primSocket: socketHandle
			sendData: aStringOrByteArray
			startIndex: bytesSent + 1
			count: (bytesToSend - bytesSent min: 5000).
		bytesSent := bytesSent + count.
		(count = 0 and: [ bytesSent ~= bytesToSend ])
			ifTrue: [ self destroy. ^ false ] ].
	^ true! !

WAScriptLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'as yet unclassified' stamp: 'lr 4/11/2006 00:15'!
aBase
	^ '/* Seaside Comet API
 *  (c) 2006 Lukas Renggli (renggli@iam.unibe.ch)
 */
Ajax.Comet = Class.create();
Ajax.Comet.prototype = {
	initialize: function(url, options) {
		this.options = options || {};
		this.iframe = Builder.node("iframe", {
			style: "position: absolute; display: none",
			location: url });
		this.updater = new PeriodicalExecuter(function() {
			if(!!this.isConnected())
				this.reconnect(); 
		}, this.options.timeout || 1000);
	},
	isConnected: function() {
		return this.iframe.contentWindow.document.readyState !!= "complete";
	},
	reconnect: function() {
		this.iframe.contentWindow.location.reload();
	}
});'! !

Object subclass: #CTNotifier
	instanceVariableNames: 'id mutex sockets process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTNotifier class methodsFor: 'instance-creation' stamp: 'lr 4/10/2006 22:56'!
new
	^ CTRegistry default register: super new! !

!CTNotifier methodsFor: 'configuration' stamp: 'lr 4/10/2006 22:46'!
delay
	^ 7500! !

!CTNotifier methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:51'!
finalize
	super finalize.
	process terminate.
	sockets do: [ :each | each destroy ]! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/11/2006 00:38'!
header
	^ String streamContents: [ :stream |
		stream nextPutAll: '<?xml version="1.0" encoding="UTF-8"?><!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml"><head><meta name="generator" content="Seaside Comet" /></head><body>'.
		[ stream contents size < 1024 ]
			whileTrue: [ stream nextPut: Character space ] ]! !

!CTNotifier methodsFor: 'accessing' stamp: 'lr 4/9/2006 21:14'!
id
	^ id! !

!CTNotifier methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:41'!
initialize
	super initialize.
	id := (WAExternalID new: 24) asString.
	mutex := Semaphore forMutualExclusion.
	sockets := Array new.
	process := [ [ self process ] repeat ]
		forkAt: self priority
		named: self name! !

!CTNotifier methodsFor: 'actions' stamp: 'lr 4/11/2006 00:25'!
notify: anObject
	self send: '<script type="text/javascript">' , anObject asJavascript , '</script>'! !

!CTNotifier methodsFor: 'printing' stamp: 'lr 4/10/2006 17:15'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' id: '; print: self id! !

!CTNotifier methodsFor: 'configuration' stamp: 'lr 4/10/2006 22:55'!
priority
	^ 10! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/11/2006 00:26'!
process
	(Delay forMilliseconds: self delay)
		wait.
	self send: ' '! !

!CTNotifier methodsFor: 'actions' stamp: 'lr 4/11/2006 00:38'!
register: aSocket
	(aSocket pushData: (String streamContents: [ :stream |
		WAResponse new
			doNotCache;
			contents: self header;
			writeOn: stream ]))
		ifFalse: [ ^ self ].
	mutex critical: [ sockets := sockets copyWith: aSocket ]! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/10/2006 22:43'!
send: aString
	mutex critical: [
		sockets := sockets select: [ :each | 
			each pushData: aString ] ]! !

Object subclass: #CTRegistry
	instanceVariableNames: 'mutex notifiers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTRegistry class
	instanceVariableNames: 'default'!

!CTRegistry class methodsFor: 'accessing' stamp: 'lr 4/10/2006 22:54'!
default
	^ default ifNil: [ default := self new ]! !

!CTRegistry class methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:57'!
initialize
	Smalltalk addToStartUpList: self after: WAKom! !

!CTRegistry class methodsFor: 'actions' stamp: 'lr 4/10/2006 22:56'!
start
	| assembly |
	assembly := ModuleAssembly core.
	assembly alias: '/comet' to: [
		assembly addPlug: (MessageSend
			receiver: self default
			selector: #process:
			argument: nil) ].
	assembly alias: '/seaside' to: [
		assembly addPlug: (MessageSend
			receiver: WAKom default
			selector: #process:
			argument: nil) ].
	WAKom service
		plug: assembly rootModule! !

!CTRegistry class methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:57'!
startUp
	default ifNotNil: [ self start ]! !

!CTRegistry class methodsFor: 'actions' stamp: 'lr 4/10/2006 22:57'!
stop
	WAKom startOn: WAKom port.
	default := nil! !

!CTRegistry methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:51'!
initialize
	super initialize.
	mutex := Semaphore forMutualExclusion.
	notifiers := WeakValueDictionary new! !

!CTRegistry methodsFor: 'processing' stamp: 'lr 4/10/2006 22:52'!
process: aRequest
	| parts handler |
	parts := aRequest url findTokens: '/'.
 	handler := mutex critical: [
		(notifiers at: parts last ifAbsent: [ ^ nil ])
			ifNil: [ ^ nil ] ].
	handler register: aRequest stream socket.
	thisContext swapSender: nil.
	Processor activeProcess terminate! !

!CTRegistry methodsFor: 'actions' stamp: 'lr 4/10/2006 22:51'!
register: aNotifier
	^ mutex critical: [ notifiers at: aNotifier id put: aNotifier ]! !
CTRegistry initialize!
